#!/usr/bin/env python
# Example uri: http://news.bbc.co.uk/rss/feeds.opml

import os
import sys
import urllib2
import sqlite3 as sqlite
from time import mktime
from datetime import datetime
from xml.sax import parse as saxparse
from xml.sax.handler import ContentHandler

db_cursor = None
epoch = None

class ProcessOutline(ContentHandler):

    def __init__(self, add_function):
        ContentHandler.__init__(self)
        self.add_function = add_function

    def startElement(self, tag, attrs):
        if tag != 'outline' or not attrs.has_key('type'):
            return

        if attrs['type'] == 'rss':
            try:
                url = repair(attrs['xmlUrl'])
            except:
                url = repair(attrs['url'])

            try:
                text = repair(attrs['text'])
            except:
                text = ''

            try:
                title = repair(attrs['title'])
            except:
                title = ''

        self.add_function ((url, text, title))


def repair(text):
    if text.find('"') != -1:
        return text.replace('"', "'")
    return text


def parse(opml_url, callback):

    if os.path.isfile (opml_url):
        opml_file = open (opml_url, 'r')

    else:
        req = urllib2.Request(opml_url)
        opml_file = urllib2.urlopen(req)

    po = ProcessOutline(callback)
    saxparse(opml_file, po)

    opml_file.close()


def add_feed(feed):

    global db_cursor
    global epoch

    xmlUrl, title, text = feed
    insert = 'INSERT INTO feeds (uri, title, desc, epoch) ' \
             'VALUES ("%s", "%s", "%s", %d)' % \
             (xmlUrl, title, text, epoch)
    try:
        db_cursor.execute (insert)
    except sqlite.IntegrityError, detail:
        print 'Feed already inserted on database.'


def main ():

    global db_cursor
    global epoch

    feedsource = sys.argv[1]

    try:
        dbfile = sys.argv[2]
    except:
        dbfile = '/home/user/.canola/canola.db'

    try:
        db = sqlite.connect (dbfile)
    except:
        print 'Problems connecting to %s.' % dbfile
        sys.exit (1)

    today = datetime.now()
    epoch = mktime((today.year, today.month, today.day - 1,
                    0, 0, 0, 0, 0, 0))

    db_cursor = db.cursor ()

    try:
        feeds = parse (feedsource, add_feed)
    except Exception, detail:
        print 'Problems to process the feed from %s.' % feedsource
        print 'Provide a valid url or opml file.'
        sys.exit (1)

    db.commit()


if __name__ == '__main__':

    if len (sys.argv) not in (2, 3):
        print 'Usage: %s opmluri [dbfile]' % sys.argv[0]
        print '\nThe default value for dbfile is "/home/user/.canola/canola.db"'
        sys.exit (1)

    main ()

